<?php

use Tygh\Storage;
use Tygh\Settings;
use Tygh\Registry;

if ( !defined('AREA') ) { die('Access denied'); }

function fn_annotation_tool_update_product_pre(&$product_data, $product_id, $lang_code, $can_update) {

    // we can access the addon settings as follows:
    // Registry::get('addons.annotation_tool.license_key')

    ini_set('max_input_vars', 10000);
    ini_set('post_max_size', '100M');
    ini_set('display_errors', 'on');

    $base_dir = explode('/app/', __DIR__);
    $base_dir = $base_dir[0] . '/';
    $tmp_img_dir = $base_dir . 'var/custom_files/';

    // new images added, check for each if it has been annotated, if yes then alter the image in /var/custom_files/sess_data
    if (isset($_POST['file_product_add_additional_image_detailed']) &&
        isset($_POST['annotated']) &&
        isset($_POST['annotated']['added'])
    ) {

        $new_img_index = 0;
        foreach ($_POST['file_product_add_additional_image_detailed'] as $new_image) {

            $img_file = $tmp_img_dir . $new_image;

            if (file_exists($img_file)) {
                $img = imagecreatefromstring(base64_decode(substr($_POST['annotated']['added'][$new_img_index], 22)));
                imagejpeg($img, $img_file, 100);
            }
            
            $new_img_index++;
        }
        
    }

    
    // check if any of the existing images have been annotated, if yes alter the image
    if (isset($_POST['annotated']) && isset($_POST['annotated']['existing'])) {
        // get main and additional images for this product
        $image_pairs = fn_get_image_pairs($product_id, 'product', 'A', FALSE, TRUE);
        $image_pairs[] = fn_get_image_pairs($product_id, 'product', 'M', FALSE, TRUE);


        // update images that were annotated
        foreach ($_POST['annotated']['existing'] as $image_id => $image_data) {

            foreach ($image_pairs as $image_pair) {
                if ($image_id == $image_pair['pair_id']) {

                    // found the image to be updated, update it
                    if (strpos($image_pair['detailed']['absolute_path'], '/watermarked/') === FALSE) {
                        $image_path = $image_pair['detailed']['absolute_path'];
                    } else {
                        $image_path = preg_replace("/\/watermarked\/\d+\//", '/', $image_pair['detailed']['absolute_path']);
                    }

                    $img = imagecreatefromstring(base64_decode(substr($image_data, 22)));
                    imagepng($img, $image_path);
                    imagepng($img, $image_pair['detailed']['absolute_path']);

                    // since this is an existing image we have to update the thumbnails too
                    // as they are already generated without the annotations
                    // all we need to do is delete the existing thumbnails
                    fn_delete_image_thumbnails($image_pair['detailed']['relative_path']);
                    fn_delete_image_thumbnails($image_pair['detailed']['relative_path'], Storage::instance('images')->options['prefix']);
                    fn_delete_image_thumbnails($image_pair['detailed']['relative_path'], 'watermarked/*/');
                    fn_delete_image_thumbnails($image_pair['detailed']['relative_path'], 'images/watermarked/*/');


                    // this is only needed if watermarks addon is activated, but doesn't do harm to be ran allways
                    $thumbnail_settings = Settings::instance()->getValues('Thumbnails');
                    fn_generate_thumbnail($image_pair['detailed']['absolute_path'], $thumbnail_settings['product_details_thumbnail_width']);

                    break;
                }
            }
        }
    }

    // cloned and annotated images
    if (isset($_POST['annotated']) && isset($_POST['annotated']['clone'])) {

        foreach ($_POST['annotated']['clone'] as $image_id => $image_data) {
            // store the image to temporary files
            $image_name = random_string() . '.png';
            $image_path = $tmp_img_dir . 'sess_data/' . $image_name;
            
            $img = imagecreatefromstring(base64_decode(substr($image_data, 22)));
            imagepng($img, $image_path);

            $image_data = array(
                'path' => $image_path,
                'name' => $image_name
            );


            //$file = fn_move_uploaded_file($image_data);

            // attach image to product
            $image_id = fn_update_image($image_data);

            $subdir = fn_get_image_subdir($image_id);

            // store a copy where needed
            imagepng($img, $base_dir . 'images/detailed/' . $subdir . '/' . $image_name);

            // add to DB - link the image to product
            db_query("INSERT INTO ?:images_links(`object_id`, `object_type`, `image_id`, `detailed_id`, `type`, `position`)
            VALUES($product_id, 'product', $image_id, $image_id, 'A', 0)");
        }
    }

    // not necessary but lessens the chance for something to go wrong in future versions of CScart
    unset($product_data['annotated']);
    unset($_POST['annotated']);

    return TRUE;

}

function random_string() {
    $chars = 'abcdefghijklmnopqrstuvwxyz';
    $chars .= strtoupper($chars);
    $chars .= '0123456789';

    $chars = str_split($chars);

    $string = '';

    for ($i = 0; $i < 10; $i++) {
        $string .= $chars[rand(0, count($chars) - 1)];
    }

    return $string;
}

function fn_annotation_tool_settings_info() {
    $ch = curl_init('https://niltaurus.com/license_info.html');
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
    $info = curl_exec($ch);
    curl_close($ch);
    return $info;
}

function fn_annotation_tool_get_route($req) {
    if ($req['dispatch'] === 'products.update') {
        $_SESSION['annotation_tool_license_key'] = Registry::get('addons.annotation_tool.license_key');
        //Tygh::$app['view']->assign('annotation_tool_license', Registry::get('addons.annotation_tool.license_key')); 
        //Registry::get('view')->assign('annotation_tool_license', Registry::get('addons.annotation_tool.license_key'));
    }
    return TRUE;
}


?>